<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/license.txt for distribution/license details
//
// user.lib.php
// User Library, provides the IUser interface


class IUser
{

// ------------------------------------------------------------ PUBLIC VARIABLES

// ------------------------------------------------------------ PRIVATE VARIABLES
var $db;

var $ID = 0;
var $Name = '';
var $Password = '';
var $RegDate = 0;
var $RegEMail = '';
var $DefaultNotify = 0;
var $EMail = '';
var $ICQ = '';
var $AIM = '';
var $YIM = '';
var $MSN = '';
var $Jabber = '';
var $LastActivity = 0;
var $LastForum = 0;
var $LastLogin = 0;
var $LastLogout = 0;
var $Signature = '';
var $BirthDay = 0;
var $BirthMonth = 0;
var $BirthYear = 0;
var $About = '';
var $Title = '';
var $Location = '';
var $Homepage = '';
var $Gender = '';
var $Avatar = '';
var $AvatarX = 0;
var $AvatarY = 0;
var $Photo = '';
var $Extra;
var $ActivateKey = '';
var $Design = -1;
var $ThreadIcons = 0;
var $Flags = 0;
	//    1: reserved (fake-avatar, disabled)
	//    2: read-line
	//    4: "hide code/quote"
	//    8: (admin) display all notifications and UserRead data
	//   16: FastReply
	//   32: Auto Re-Login (with Cookies)
	//   64: include a copy of the post on notification
	//  128: Open links to _blank
	//  256: NEWLAYOUT
	//  512: Half Size Avatars
	// 1024: Hide avatars
	// 2048: Hide signatures
var $Timezone = 0;
	// +/- offset (hours) * 4 (-48...+48)
	// 99: board default
var $TimezoneDS = -1;
	// daylight saving time? (-1 board default, 0 false, 1 auto, 2 force)
var $EditControls = 0;
	// 0: board default
var $ThreadsPerPage = 0;
var $ThreadSort = 0;
var $ThreadTime = 0;
var $Favourites = '';
	// space-separated list of Thread IDs
var $Language = '';
	// ISO code (de, en, fr...)
var $DateFormat = '';


// ------------------------------------------------------------ CONSTRUCTOR

function IUser()
{
	global $extra_count, $db;

	if (PHP5) eval('$this->db = clone $db;');
	else      $this->db = $db;

	if ($extra_count > 0)
		$this->Extra = array_fill(1, $extra_count, '');
	else
		$this->Extra = array();
}

function Reset()
{
	$this->ID = 0;
	$this->Name = '';
	$this->Password = '';
	$this->RegDate = 0;
	$this->RegEMail = '';
	$this->DefaultNotify = 0;
	$this->EMail = '';
	$this->ICQ = '';
	$this->AIM = '';
	$this->YIM = '';
	$this->MSN = '';
	$this->Jabber = '';
	$this->LastActivity = 0;
	$this->LastForum = 0;
	$this->LastLogin = 0;
	$this->LastLogout = 0;
	$this->Signature = '';
	$this->BirthDay = 0;
	$this->BirthMonth = 0;
	$this->BirthYear = 0;
	$this->About = '';
	$this->Title = '';
	$this->Location = '';
	$this->Homepage = '';
	$this->Gender = '';
	$this->Avatar = '';
	$this->AvatarX = 0;
	$this->AvatarY = 0;
	$this->Photo = '';
	$this->Extra = array();
	$this->ActivateKey = '';
	$this->Design = -1;
	$this->ThreadIcons = 0;
	$this->Flags = 0;
	$this->Timezone = 0;
	$this->TimezoneDS = 0;
	$this->EditControls = 0;
	$this->ThreadsPerPage = 0;
	$this->ThreadSort = 0;
	$this->ThreadTime = 0;
	$this->Favourites = '';
	$this->Language = '';
	$this->DateFormat = '';
}


// ------------------------------------------------------------ FIND

// queries a list of some/all users
//
function GetList($where = '', $order = '', $limit = '')
{
	$record = $this->db->FastQuery('Users', '*', $where, $order, $limit);
	return $this->LoadFromRecord($record);
}

function GetNext()
{
	$record = $this->db->GetRecord();
	return $this->LoadFromRecord($record);
}

// Get an array(ID) of UserIDs matching the query
//
function GetListArray($where = '', $order = '', $limit = '')
{
	return $this->db->FastQuery1stArray('Users', 'ID', $where, $order, $limit);
}


// Find a user by name
//
function FindByName($name)
{
	$name = db_encode($name);
	if ($this->GetList('Name LIKE "' . $name . '"'))
	{
		return $this->GetID();
	}
	elseif (is_numeric($name) && $this->GetList('ID=' . $name))
	{
		return $this->GetID();
	}
	else
	{
		return 0;
	}
}

// Count users
//
function Count($where = '')
{
	return $this->db->FastQuery1st('Users', 'count(*)', $where);
}


// Like Count(), but filters by UserGroups, too
//
function CountWithGroup($where = '', $group = 0, $invgroup = false)
{
	if (!$group) return $this->Count($where);

	$pre = $this->db->tblprefix;

	if ($where != '') $where = 'WHERE ' . $where . ' ';

	if ($group == -1)
	{
		$group_join = "LEFT JOIN {$pre}GroupMembers ON {$pre}Users.ID = {$pre}GroupMembers.User ";
		if ($where == '') $where .= 'WHERE '; else $where .= 'AND ';
		$where .= "{$pre}GroupMembers.Group IS " . ($invgroup ? 'NOT ' : '') . 'NULL ';
	}
	elseif ($group > 0 && !$invgroup)
	{
		$group_join = "JOIN {$pre}GroupMembers ON {$pre}Users.ID = {$pre}GroupMembers.User " .
			"AND {$pre}GroupMembers.Group = $group ";
	}
	elseif ($group > 0 && $invgroup)
	{
		$group_join = "LEFT JOIN {$pre}GroupMembers ON {$pre}Users.ID = {$pre}GroupMembers.User " .
			"AND {$pre}GroupMembers.Group = $group ";
		$where .= ($where ? ' AND ' : 'WHERE ') . "{$pre}GroupMembers.Group IS NULL ";
	}

	$this->db->Exec("SELECT COUNT(DISTINCT {$pre}Users.ID) " .
		"FROM {$pre}Users " .
			$group_join .
		$where);
	$a = $this->db->GetRecord();
	if ($a === false) return 0;
	return $a[0];
}


// ------------------------------------------------------------ READ ACCESS

function Load($id)
{
	$id = intval($id);
	if (!$id)
	{
		$this->Reset();
		return false;
	}
	global $user_cache;

	if (array_key_exists($id, $user_cache))
	{
		$record = $user_cache[$id];
		$add = false;
	}
	else
	{
		$record = $this->db->FastQuery('Users', '*', 'ID=' . $id);
		$add = true;
	}

	return $this->LoadFromRecord($record, $add);
}


function LoadFromRecord($record, $add = false)
{
	global $extra_count, $user_cache;

	if ($record)
	{
		$this->ID = intval($record['ID']);
		$this->Name = $record['Name'];
		$this->Password = $record['Password'];
		$this->RegDate = intval($record['RegDate']);
		$this->RegEMail = $record['RegEMail'];
		$this->DefaultNotify = intval($record['DefaultNotify']);
		$this->EMail = $record['EMail'];
		$this->ICQ = $record['ICQ'];
		$this->AIM = $record['AIM'];
		$this->YIM = $record['YIM'];
		$this->MSN = $record['MSN'];
		$this->Jabber = $record['Jabber'];
		$this->LastActivity = intval($record['LastActivity']);
		$this->LastForum = intval($record['LastForum']);
		$this->LastLogin = intval($record['LastLogin']);
		$this->LastLogout = intval($record['LastLogout']);
		$this->Signature = $record['Signature'];
		$this->BirthDay = intval($record['BirthDay']);
		$this->BirthMonth = intval($record['BirthMonth']);
		$this->BirthYear = intval($record['BirthYear']);
		$this->About = $record['About'];
		$this->Title = $record['Title'];
		$this->Location = $record['Location'];
		$this->Homepage = $record['Homepage'];
		$this->Gender = $record['Gender'];
		$this->Avatar = $record['Avatar'];
		$this->AvatarX = $record['AvatarX'];
		$this->AvatarY = $record['AvatarY'];
		$this->Photo = $record['Photo'];
		for ($n = 1; $n <= $extra_count; $n++)
			$this->Extra[$n] = $record['Extra' . $n];
		$this->ActivateKey = $record['ActivateKey'];
		$this->Design = intval($record['Design']);
		$this->ThreadIcons = intval($record['ThreadIcons']);
		$this->Flags = intval($record['Flags']);
		$this->Timezone = intval($record['Timezone']);
		$this->TimezoneDS = intval($record['TimezoneDS']);
		$this->EditControls = intval($record['EditControls']);
		$this->ThreadsPerPage = intval($record['ThreadsPerPage']);
		$this->ThreadSort = intval($record['ThreadSort']);
		$this->ThreadTime = intval($record['ThreadTime']);
		$this->Favourites = $record['Favourites'];
		$this->Language = $record['Language'];
		$this->DateFormat = $record['DateFormat'];

		if ($add) $user_cache[$this->ID] = $record;

		return true;
	}
	else
	{
		$this->Reset();
		return false;
	}
}


function GetID()
{
	return $this->ID;
}

function GetName($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Name;
}

function GetPassword($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Password;
}

function GetRegDate($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->RegDate;
}

function GetRegEMail($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->RegEMail;
}

function GetDefaultNotify($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->DefaultNotify;
}

function GetEMail($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->EMail;
}

function GetICQ($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->ICQ;
}

function GetAIM($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->AIM;
}

function GetYIM($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->YIM;
}

function GetMSN($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->MSN;
}

function GetJabber($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Jabber;
}

// only return a real jabber id. "uin:########" will be suppressed
function GetRealJabber($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	if (!strncmp($this->Jabber, 'uin:', 4)) return '';
	return $this->Jabber;
}

function GetLastActivity($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->LastActivity;
}

function GetLastForum($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->LastForum;
}

function GetLastLogin($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->LastLogin;
}

function GetLastLogout($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;

	$time = $this->LastLogout;
	if (!$time) $time = $this->LastLogin;
	if (!$time) $time = time();

	return $time;
}

function GetSignature($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Signature;
}

function GetBirthDay($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->BirthDay;
}

function GetBirthMonth($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->BirthMonth;
}

function GetBirthYear($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->BirthYear;
}

function GetAbout($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->About;
}

function GetTitle($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Title;
}

function GetLocation($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Location;
}

function GetHomepage($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Homepage;
}

function GetGender($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Gender;
}

function GetAvatar($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Avatar;
}

function GetAvatarX($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->AvatarX;
}

function GetAvatarY($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->AvatarY;
}

function GetPhoto($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Photo;
}

function GetExtra($n, $id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Extra[$n];
}

function GetActivateKey($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->ActivateKey;
}

function GetDesign($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Design;
}

function GetThreadIcons($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->ThreadIcons;
}

function GetFlags($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Flags;
}

function GetTimezone($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Timezone;
}

function GetTimezoneDS($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->TimezoneDS;
}

function GetEditControls($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->EditControls;
}

function GetThreadsPerPage($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->ThreadsPerPage;
}

function GetThreadSort($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->ThreadSort;
}

function GetThreadTime($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->ThreadTime;
}

function GetFavourites($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Favourites;
}

function GetLanguage($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->Language;
}

function GetDateFormat($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return $this->DateFormat;
}


function GetBirthdate($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return mktime(0, 0, 0, $this->BirthMonth, $this->BirthDay, $this->BirthYear);
}

function GetOnline($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return (time() - $this->LastActivity <= 5 * 60);    # user did something within the last 5 mins

	# update this time interval in users.php, ListOnlineUsers(), too
	# TODO: make this timespan global-configurable
}

function GetGenderVerbose($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return GetGenderVerbose($this->Gender);
}

function AvatarFromURL($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return (strpos($this->Avatar, ':'));
}

function PhotoFromURL($id = 0)
{
	if ($id != 0) if (!$this->Load($id)) return false;
	return (strpos($this->Photo, ':'));
}


// ------------------------------------------------------------ WRITE ACCESS

// Add a new user
// $Password must be passed readable, MD5 will be generated here
//
function Add($Name, $Password, $EMail)
{
	global $extra_count, $LANG;

	$max = intval($this->db->FastQuery1st('Users', 'MAX(ID)'));

	$this->ID = $max + 1;
	$this->Name = trim($Name);
	$this->Password = md5($Password);
	$this->RegDate = time();
	$this->RegEMail = trim($EMail);
	$this->DefaultNotify = 0;
	$this->EMail = trim($EMail);
	$this->ICQ = '';
	$this->AIM = '';
	$this->YIM = '';
	$this->MSN = '';
	$this->Jabber = '';
	$this->LastActivity = 0;
	$this->LastForum = 0;
	$this->LastLogin = 0;
	$this->LastLogout = 0;
	$this->Signature = '';
	$this->BirthDay = 0;
	$this->BirthMonth = 0;
	$this->BirthYear = 0;
	$this->About = '';
	$this->Title = '';
	$this->Location = '';
	$this->Homepage = '';
	$this->Gender = '';
	$this->Avatar = '';
	$this->Photo = '';
	if ($extra_count > 0)
		$this->Extra = array_fill(1, $extra_count, '');
	else
		$this->Extra = array();
	$this->ActivateKey = '';
	$this->Design = -1;   // TODO: copy from currently selected design, if this is not 'default'
	$this->ThreadIcons = 0;
	$this->Flags = 2 + 4 + 32 + 128 + (rc('new_layout') ? 256 : 0);
	$this->Timezone = 99;
	$this->TimezoneDS = -1;
	$this->EditControls = 0;
	$this->ThreadsPerPage = -1;
	$this->ThreadSort = -1;
	$this->ThreadTime = -1;
	$this->Favourites = '';
	$this->Language = $LANG;   // TODO: copy from currently selected language, if this is not 'default'
	$this->DateFormat = '';

	$add = array(
		'ID' => $this->ID,
		'Name' => $this->Name,
		'Password' => $this->Password,
		'RegDate' => $this->RegDate,
		'RegEMail' => $this->RegEMail,
		'DefaultNotify' => $this->DefaultNotify,
		'EMail' => $this->EMail,
		'ICQ' => $this->ICQ,
		'AIM' => $this->AIM,
		'YIM' => $this->YIM,
		'MSN' => $this->MSN,
		'Jabber' => $this->Jabber,
		'LastActivity' => $this->LastActivity,
		'LastForum' => $this->LastForum,
		'Signature' => $this->Signature,
		'BirthDay' => $this->BirthDay,
		'BirthMonth' => $this->BirthMonth,
		'BirthYear' => $this->BirthYear,
		'About' => $this->About,
		'Title' => $this->Title,
		'Location' => $this->Location,
		'Homepage' => $this->Homepage,
		'Gender' => $this->Gender,
		'Avatar' => $this->Avatar,
		'Photo' => $this->Photo,
		'ActivateKey' => $this->ActivateKey,
		'Design' => $this->Design,
		'ThreadIcons' => $this->ThreadIcons,
		'Flags' => $this->Flags,
		'Timezone' => $this->Timezone,
		'TimezoneDS' => $this->TimezoneDS,
		'EditControls' => $this->EditControls,
		'ThreadsPerPage' => $this->ThreadsPerPage,
		'ThreadSort' => $this->ThreadSort,
		'ThreadTime' => $this->ThreadTime,
		'Favourites' => $this->Favourites,
		'Language' => $this->Language,
		'DateFormat' => $this->DateFormat);

	for ($n = 1; $n <= $extra_count; $n++)
		$add['Extra' . $n] = $this->Extra[$n];

	$ok = $this->db->AddRecord($add, 'Users');

	// update statistics table, no error detection here
	if ($ok) UpdateStat('NewUsers', 1);
	return $ok;
}


function SetName($Name, $id = 0)
{
	global $user_cache;

	$Name = trim($Name);
	if (!$id)
	{
		$id = $this->ID;
		$this->Name = $Name;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Name' => $Name), 'ID=' . $id, 'Users');
}

function SetPassword($Password, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Password = $Password;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Password' => $Password), 'ID=' . $id, 'Users');
}

function SetDefaultNotify($DefaultNotify, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->DefaultNotify = intval($DefaultNotify);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('DefaultNotify' => $DefaultNotify), 'ID=' . $id, 'Users');
}

function SetEMail($EMail, $id = 0)
{
	global $user_cache;

	$EMail = trim($EMail);
	if (!$id)
	{
		$id = $this->ID;
		$this->EMail = $EMail;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('EMail' => $EMail), 'ID=' . $id, 'Users');
}

function SetIM($ICQ, $AIM, $YIM, $MSN, $Jabber, $id = 0)
{
	global $user_cache;

	$ICQ = trim($ICQ);
	$AIM = trim($AIM);
	$YIM = trim($YIM);
	$MSN = trim($MSN);
	$Jabber = trim($Jabber);
	if (!$id)
	{
		$id = $this->ID;
		$this->ICQ = $ICQ;
		$this->AIM = $AIM;
		$this->YIM = $YIM;
		$this->MSN = $MSN;
		$this->Jabber = $Jabber;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('ICQ' => $ICQ, 'AIM' => $AIM, 'YIM' => $YIM, 'MSN' => $MSN, 'Jabber' => $Jabber), 'ID=' . $id, 'Users');
}

function SetLastActivity($forumid, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->LastActivity = time();
		$this->LastForum = intval($forumid);
	}

	// generally we don't need to remove the cache in this case, except for who-is-where list
	if ($ME == $MODULE['users']) unset($user_cache[$id]);

	return $this->db->ChangeRecord(array('LastActivity' => time(), 'LastForum' => $forumid), 'ID=' . $id, 'Users');
}

function SetLastLogin()
{
	global $user_cache;

	$id = $this->ID;
	$this->LastLogout = $this->LastActivity;   // attention: update this BEFORE setting new LastActivity value!
	$this->LastLogin = time();

	unset($user_cache[$id]);
	$ok = $this->db->ChangeRecord(array('LastLogin' => $this->LastLogin, 'LastLogout' => $this->LastLogout), 'ID=' . $id, 'Users');

	// update statistics table, no error detection here
	$this->SetLastActivity(0);   // set Active time now, so that Users will be counted correctly for UpdateStat
	if ($ok) UpdateUserStat();
	return $ok;
}

function SetSignature($Signature, $id = 0)
{
	global $user_cache;

	$Signature = trim($Signature);
	if (!$id)
	{
		$id = $this->ID;
		$this->Signature = $Signature;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Signature' => $Signature), 'ID=' . $id, 'Users');
}

function SetBirthDate($BirthDay, $BirthMonth, $BirthYear, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->BirthDay = intval($BirthDay);
		$this->BirthMonth = intval($BirthMonth);
		$this->BirthYear = intval($BirthYear);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('BirthDay' => $BirthDay, 'BirthMonth' => $BirthMonth, 'BirthYear' => $BirthYear), 'ID=' . $id, 'Users');
}

function SetAbout($About, $id = 0)
{
	global $user_cache;

	$About = trim($About);
	if (!$id)
	{
		$id = $this->ID;
		$this->About = $About;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('About' => $About), 'ID=' . $id, 'Users');
}

function SetTitle($Title, $id = 0)
{
	global $user_cache;

	$Title = trim($Title);
	if (!$id)
	{
		$id = $this->ID;
		$this->Title = $Title;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Title' => $Title), 'ID=' . $id, 'Users');
}

function SetHomepage($Homepage, $id = 0)
{
	global $user_cache;

	$Homepage = trim($Homepage);
	if (!$id)
	{
		$id = $this->ID;
		$this->Homepage = $Homepage;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Homepage' => $Homepage), 'ID=' . $id, 'Users');
}

function SetLocation($Location, $id = 0)
{
	global $user_cache;

	$Location = trim($Location);
	if (!$id)
	{
		$id = $this->ID;
		$this->Location = $Location;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Location' => $Location), 'ID=' . $id, 'Users');
}

function SetGender($Gender, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Gender = $Gender;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Gender' => $Gender), 'ID=' . $id, 'Users');
}

function SetAvatar($Avatar)
{
	global $user_cache, $avatar_path;

	if ($Avatar == '' && $this->Avatar == '') return true;

	$id = $this->ID;
	if ($Avatar == '')
	{
		// remove image file from disk, too (if it exists, anyway)
		if (file_exists($avatar_path . $this->Avatar))
			if (!unlink($avatar_path . $this->Avatar)) return false;

		$newx = $newy = 0;
	}

	$this->Avatar = $Avatar;
	unset($user_cache[$id]);

	if ($Avatar != '')
	{
		$is = @getimagesize((strpos($Avatar, ':') ? '' : $avatar_path) . $Avatar);
		if (is_array($is))
		{
			$newx = $is[0];
			$newy = $is[1];
		}
		else
		{
			$newx = $newy = 0;
		}
	}

	return $this->db->ChangeRecord(array('Avatar' => $Avatar, 'AvatarX' => $newx, 'AvatarY' => $newy), 'ID=' . $id, 'Users');
}

function SetPhoto($Photo)
{
	global $user_cache, $photo_path;

	if ($Photo == '' && $this->Photo == '') return true;

	$id = $this->ID;
	if ($Photo == '')
	{
		// remove image file from disk, too (if it exists, anyway)
		if (file_exists($photo_path . $this->Photo))
			if (!unlink($photo_path . $this->Photo)) return false;
	}

	$this->Photo = $Photo;
	unset($user_cache[$id]);

	return $this->db->ChangeRecord(array('Photo' => $Photo), 'ID=' . $id, 'Users');
}

function SetExtra($n, $Extra, $id = 0)
{
	global $user_cache;

	$Extra = trim($Extra);
	if (!$id)
	{
		$id = $this->ID;
		$this->Extra[$n] = $Extra;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Extra' . $n => $Extra), 'ID=' . $id, 'Users');
}

function SetActivateKey($ActivateKey, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->ActivateKey = $ActivateKey;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('ActivateKey' => $ActivateKey), 'ID=' . $id, 'Users');
}

function SetDesign($Design, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Design = intval($Design);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Design' => $Design), 'ID=' . $id, 'Users');
}

function SetThreadIcons($ThreadIcons, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->ThreadIcons = intval($ThreadIcons);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('ThreadIcons' => $ThreadIcons), 'ID=' . $id, 'Users');
}

function SetFlags($Flags, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Flags = intval($Flags);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Flags' => $Flags), 'ID=' . $id, 'Users');
}

function SetTimezone($Timezone, $TimezoneDS, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Timezone = intval($Timezone);
		$this->TimezoneDS = intval($TimezoneDS);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Timezone' => $Timezone, 'TimezoneDS' => $TimezoneDS), 'ID=' . $id, 'Users');
}

function SetEditControls($EditControls, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->EditControls = intval($EditControls);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('EditControls' => $EditControls), 'ID=' . $id, 'Users');
}

function SetThreadsPerPage($ThreadsPerPage, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->ThreadsPerPage = intval($ThreadsPerPage);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('ThreadsPerPage' => $ThreadsPerPage), 'ID=' . $id, 'Users');
}

function SetThreadSort($ThreadSort, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->ThreadSort = intval($ThreadSort);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('ThreadSort' => $ThreadSort), 'ID=' . $id, 'Users');
}

function SetThreadTime($ThreadTime, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->ThreadTime = intval($ThreadTime);
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('ThreadTime' => $ThreadTime), 'ID=' . $id, 'Users');
}

function SetFavourites($Favourites, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Favourites = $Favourites;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Favourites' => $Favourites), 'ID=' . $id, 'Users');
}

function SetLanguage($Language, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->Language = $Language;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('Language' => $Language), 'ID=' . $id, 'Users');
}

function SetDateFormat($DateFormat, $id = 0)
{
	global $user_cache;

	if (!$id)
	{
		$id = $this->ID;
		$this->DateFormat = $DateFormat;
	}
	unset($user_cache[$id]);
	return $this->db->ChangeRecord(array('DateFormat' => $DateFormat), 'ID=' . $id, 'Users');
}


function Remove($id = 0)
{
	global $user_cache, $LoginUserID, $avatar_path, $photo_path;

	if ($id) if (!$this->Load($id)) return false;
	$id = $this->ID;

	// noone may delete himself!
	if ($id == $LoginUserID) return false;

	// remove avatar & photo image file
	if ($this->Avatar != '' && file_exists($avatar_path . $this->Avatar))
		if (!unlink($avatar_path . $this->Avatar)) return false;
	if ($this->Photo != '' && file_exists($photo_path . $this->Photo))
		if (!unlink($photo_path . $this->Photo)) return false;

	// remove any records from some tables relating to (this) user
	// UserCategorySate
	$forum = new IForum;
	$forum->RemoveAllUserStates($id);
	// ThreadWatch
	$thread = new IThread;
	$thread->RemoveAllUserWatchs($id);
	// ForumWatch
	$forum->RemoveAllUserWatchs($id);
	// PollUsers
	$thread->RemoveAllUserVotes($id);
	// MessageRead
	$message = new IMessage;
	$message->RemoveAllUserReads($id);
	// GroupMembers
	SetUserGroups($id, array());
	// ACL
	$this->db->RemoveRecord('User=' . $id, 'ACL');

	// update posts created by this user
	$post = new IPost;
	if ($post->Find('User=' . $id)) do
	{
		if (!$post->SetUser(-1, $this->Name)) echo 'error setting post user/name<br />';
	}
	while ($post->FindNext());

	// update posts changed by this user
	if ($post->Find('EditUser=' . $id)) do
	{
		if (!$post->SetUser(false, false, 0)) echo 'error setting post edituser<br />';
	}
	while ($post->FindNext());

	// update threads created by this user
	$thread = new IThread;
	if ($thread->Find('User=' . $id)) do
	{
		if (!$thread->SetUser(-1, $this->Name)) echo 'error setting thread user/name<br />';
	}
	while ($thread->FindNext());

	unset($user_cache[$id]);
	$ok = $this->db->RemoveRecord('ID=' . $id, 'Users');

	// update statistics table, no error detection here
	if ($ok) UpdateStat('NewUsers', -1);
	return $ok;
}


// Extracted from GetThreadWatchs()
function build_forder($parent, &$forder_a, $forums_tbl)
{
	foreach ($forums_tbl as $f)
	{
		if ($f['Parent'] == $parent)
		{
			//if (in_array($f['ID'], $forder_a)) continue;   // Loop found! [disabled for performance]
			array_push($forder_a, $f['ID']);
			$this->build_forder($f['ID'], $forder_a, $forums_tbl);
		}
	}
}

// Get all ThreadWatchs for (this) user
//
function GetThreadWatchs($id = 0, $cond = '')
{
	$pre = $this->db->tblprefix;

	// Determine global order of forums

	$forums_tbl = $this->db->FastQueryArray('Forums', 'ID, Parent', '', 'Sort, Name');
	if ($forums_tbl === false || $forums_tbl == array()) return array();

	$forder_a = array();
	$this->build_forder(0, $forder_a, $forums_tbl);

	$forder = "CASE {$pre}Forums.ID ";

	foreach ($forder_a as $pos => $fid)
	{
		$forder .= 'WHEN ' . $fid . ' THEN ' . $pos . ' ';
	}
	$forder .= 'END';

	if ($cond == '') $cond = '>0';
	if (!$id) $id = $this->ID;
	$this->db->Exec("SELECT {$pre}ThreadWatch.Thread, {$pre}ThreadWatch.Mode, {$pre}Threads.LastPostDate, {$pre}Threads.Forum, {$pre}Threads.Subject " .
		"FROM {$pre}Threads " .
			"LEFT JOIN {$pre}ThreadWatch ON {$pre}ThreadWatch.Thread={$pre}Threads.ID " .
			"LEFT JOIN {$pre}Forums ON {$pre}Threads.Forum={$pre}Forums.ID " .
		"WHERE {$pre}ThreadWatch.User=$id AND {$pre}ThreadWatch.Mode$cond " .
		"ORDER BY {$forder}, {$pre}Threads.LastPostDate DESC");
	$arr = array();
	while ($record = $this->db->GetRecord())
	{
		$record['Mode'] = intval($record['Mode']);
		array_push($arr, $record);
	}
	return $arr;
	//return $this->db->FastQueryArray("ThreadWatch", "Thread, Mode", "User=$id and Mode>0", "Thread");
}

}  // class

?>