<?php
// This is a little tool to display the current user's uid and that of all
// files in the current directory. It's a small part of WMS' file manager.
// It may be helpful to discover uid-related problems writing to files at
// some mysterious webhoster. This program is distributed as part of the
// BlackBoard Internet Newsboard System (http://software.unclassified.de/bb).

// add or remove the directory name trailing slash as needed
function trailing_slash($path, $add = true)
{
	if (substr($path, strlen($path) - 1, 1) != '/' && $add) $path .= '/';
	if (substr($path, strlen($path) - 1, 1) == '/' && !$add) $path = substr($path, 0, strlen($path) - 1);
	return $path;
}

// current working directory
$rootdir = $_SERVER['DOCUMENT_ROOT'];   // root directory
$rootdir = str_replace('\\', '/', $rootdir);   // replace windows' \ path delimiters by unix' /
if (substr($rootdir, strlen($rootdir) - 1) == '/') $rootdir = substr($rootdir, 0, strlen($rootdir) - 1);   // remove trailing slash
$cwd     = $rootdir . '/';   // absolute current working directory, with trailing slash

if (isset($_REQUEST['cwd'])) $cwd .= $_REQUEST['cwd'];
$cwd = trailing_slash($cwd);

// file flags
define('FM_NONE', 0);
define('FM_DIR',  1);

// Text-to-HTML conversion
// Masks special HTML characters: & < >
// Unicode-Safe (optional: $unicode): "&#...;" will not be replaced since this is
//     the alternative description of Unicode characters
// if ($spaces), multiple subsequent " " will be masked in a way that no space character gets lost
//
function t2h($text, $spaces = true, $quotes = true, $nl2br = false, $unicode = false)
{
	if ($unicode) $text = preg_replace('/&(?!#\d+?;)/', '&amp;', $text);
	else $text = preg_replace('/&/', '&amp;', $text);
	$text = preg_replace('/</', '&lt;', $text);
	$text = preg_replace('/>/', '&gt;', $text);
	if ($quotes) $text = str_replace('"', '&quot;', $text);

	// multiple spaces and tabs
	if ($spaces)
	{
		$text = preg_replace('/\t/', '    ', $text);
		$text = preg_replace('/  /', '&nbsp; ', $text);
		$text = preg_replace('/  /', '&nbsp; ', $text);
	}

	if ($nl2br)
	{
		$text = nl2br($text);
		$text = preg_replace('~<br />~', '<br>', $text);
	}

	return $text;
}

// formats a number
//
// x = number for format
// decimals = maximum number of decimals | false
// range_factor = 1000 | 1024...
// sep = separator between number and range sign (will always be added)
//
function format_number($x, $decimals = false, $range_factor = 0, $sep = '')
{
	if ($x === '') return '';
	if (!is_numeric($x)) return $x;

	$decimal = ',';
	$thousand = '.';
	$exp_names = array(-4 => 'p', -3 => 'n', -2 => '', -1 => 'm', 0 => '', 1 => 'k', 2 => 'M', 3 => 'G', 4 => 'T', 5 => 'P');

	// adjust range, if requested
	$exp = 0;
	if ($range_factor && $x)
	{
		if ($range_factor != 1024)
		{
			// 1024 is likely to adjust bytes' range. don't go below 1 in that case...
			// else:
			while ($x < 0.0012 * $range_factor)
			{
				$x *= $range_factor;
				$exp--;
			}
		}
		while ($x > 1.2 * $range_factor)
		{
			$x /= (float) $range_factor;
			$exp++;
		}
	}
	$exp = $exp_names[$exp];

	// limit decimals
	if ($decimals > 0) $x = round($x, $decimals);

	// split up number into integer and decimal parts
	list($int, $dec) = explode('.', $x);

	// insert thousands separators
	$a = '';
	for ($n = 0; $n < strlen($int); $n++)
	{
		if (($n % 3 == 0) && ($n > 0) && ($n < strlen($int))) $a = $thousand . $a;
		$a = substr($int, strlen($int) - 1 - $n, 1) . $a;
	}
	if ($a == '') $a = '0';

	// re-combine with correct decimal separator
	if ($decimals === false)
	{
		if ($dec != '') $a .= $decimal . $dec;
	}
	elseif ($decimals > 0)
	{
		#if (strlen($dec) < $decimals) $dec = str_pad($dec, $decimals, '0');
		if (strlen($dec) && intval($dec)) $a .= $decimal . $dec;
	}

	if ($sep != '') $a .= $sep;
	if ($exp != '') $a .= $exp;

	return $a;
}

// translate file permission number in verbose string
//
function fileperms_to_str($in_Perms)
{
	if ($in_Perms === 'ERR') return $in_Perms;

	if ($in_Perms & 0x1000)     // FIFO pipe
		$sP = 'p';
	elseif ($in_Perms & 0x2000) // Character special
		$sP = 'c';
	elseif ($in_Perms & 0x4000) // Directory
		$sP = 'd';
	elseif ($in_Perms & 0x6000) // Block special
		$sP = 'b';
	elseif ($in_Perms & 0x8000) // Regular
		$sP = '-';
	elseif ($in_Perms & 0xA000) // Symbolic Link
		$sP = 'l';
	elseif ($in_Perms & 0xC000) // Socket
		$sP = 's';
	else                        // UNKNOWN
		$sP = 'u';

	// owner
	$sP .= (($in_Perms & 0x0100) ? 'r' : '-') .
		   (($in_Perms & 0x0080) ? 'w' : '-') .
		   (($in_Perms & 0x0040) ? (($in_Perms & 0x0800) ? 's' : 'x' ) :
								   (($in_Perms & 0x0800) ? 'S' : '-'));

	// group
	$sP .= (($in_Perms & 0x0020) ? 'r' : '-') .
		   (($in_Perms & 0x0010) ? 'w' : '-') .
		   (($in_Perms & 0x0008) ? (($in_Perms & 0x0400) ? 's' : 'x' ) :
								   (($in_Perms & 0x0400) ? 'S' : '-'));

	// world
	$sP .= (($in_Perms & 0x0004) ? 'r' : '-') .
		   (($in_Perms & 0x0002) ? 'w' : '-') .
		   (($in_Perms & 0x0001) ? (($in_Perms & 0x0200) ? 't' : 'x' ) :
								   (($in_Perms & 0x0200) ? 'T' : '-'));

	return $sP;
}

// returns an array(name, size, time) of files in a directory
// or false on error
//
function get_files($mycwd = '')
{
	global $cwd;
	if ($mycwd == '') $mycwd = $cwd;

	$files = array();

	@$handle = opendir($mycwd);
	if ($handle === false) return false;

	while ($file = readdir($handle))
	{
		$flags = FM_NONE;
		if (is_dir($mycwd . $file))
		{
			$size = '';
			$flags |= FM_DIR;
		}
		else
		{
			$size = @filesize($mycwd . $file);
			if ($size === false) $size = 'ERR';
		}

		$time = @filemtime($mycwd . $file);
		if ($time === false) $time = 'ERR';

		$s = @stat($mycwd . $file);
		if ($s === false) $s = array('uid' => 'ERR', 'gid' => 'ERR');

		$perm = @fileperms($mycwd . $file);
		if ($perm === false) $perm = 'ERR';

		$link = '';
		if (is_link($mycwd . $file)) $link = readlink($mycwd . $file);

		array_push($files, array(
			'name' => $file,
			'size' => $size,
			'time' => $time,
			'flags' => $flags,
			'uid' => $s['uid'],
			'gid' => $s['gid'],
			'mode' => $perm,
			'link' => $link
			));
	}

	closedir($handle);

	return $files;
}

// sort the filename array
// output contains directories first, then all files
//
// $by: name, size, time
// $order: asc, desc
//
function sort_files(&$all, $by = 'name', $order = 'asc')
{
	$asc = ($order == 'asc');

	// split array in directories and files
	$sub = array();   // 0=dirs, 1=files
	$sub[0] = array();
	$sub[1] = array();

	foreach ($all as $one)
	{
		if ($one['flags'] & FM_DIR)
			array_push($sub[0], $one);
		else
			array_push($sub[1], $one);
	}

	// sort directories and files
	for ($x = 0; $x <= 1; $x++)   // dirs/files
	{
		for ($a = 0; $a < sizeof($sub[$x]) - 1; $a++)   // first element to compare
		{
			for ($b = $a + 1; $b < sizeof($sub[$x]); $b++)   // second element to compare
			{
				if ($asc  && ($sub[$x][$a][$by] > $sub[$x][$b][$by]) ||
				    !$asc && ($sub[$x][$a][$by] < $sub[$x][$b][$by]))
				{
					// swap both elements
					$swap = $sub[$x][$a];
					$sub[$x][$a] = $sub[$x][$b];
					$sub[$x][$b] = $swap;
				}
			}
		}
	}

	$all = array_merge($sub[0], $sub[1]);

	return true;
}

// get user/group name from uid/gid
//
function username_from_uid($uid)
{
	if ($uid === 'ERR') return $uid;
	if (!function_exists('posix_getpwuid')) return '-';
	$a = @posix_getpwuid($uid);
	if ($a === false) return '#' . $uid;
	return $a['name'];
}

function groupname_from_gid($gid)
{
	if ($gid === 'ERR') return $gid;
	if (!function_exists('posix_getgrgid')) return '-';
	$a = @posix_getgrgid($gid);
	if ($a === false) return '#' . $gid;
	return $a['name'];
}

$mycwd = $cwd;

$myuid = getmyuid();
$mygid = getmygid();

echo "<html>
<head>
<style type='text/css'>
th
{
	font: 9pt Andale Mono, Courier New;
	text-align: left;
	color: #A00000;
}
td
{
	font: 9pt Andale Mono, Courier New;
	background: #EEEEEE;
}
</style>
</head>

<body>
<p>
Aktueller Benutzer: " . t2h(username_from_uid($myuid) . ' (' . $myuid . ')') . ", Gruppe: " . t2h(groupname_from_gid($mygid) . ' (' . $mygid . ')') . "<br>
'id' sagt: " . t2h(exec('id')) . "<br>
Aktuelle umask: " . sprintf('%03o', umask()) . "<br>
Aktuelles Verzeichnis: " . t2h($mycwd) . "
</p>

<p>
<table width=100% border=0 cellspacing=1 cellpadding=1>
<tr>
	<th>Dateiname</th>

	<th width=90>Mode</th>
	<th width=95>User</th>
	<th width=95>Group</th>

	<th width=100>Gre</th>
	<th width=145>nderung</th>
</tr>";

$files = get_files();
if (is_array($files))
{
	sort_files($files);

	foreach ($files as $file)
	{
		$name = $file['name'];
		if ($file['flags'] & FM_DIR) $name = t2h($name) . '/';
		else                         $name = t2h($name);

		$link = '';
		if ($file['link'] != '') $link = ' -&gt; ' . t2h($file['link']);

		echo '<tr>';
		echo '<td><nobr>' . $name . $link . '</nobr></td>';

		echo '<td><nobr>' . fileperms_to_str($file['mode']) . '</nobr></td>';
		$s = @stat($mycwd . '..');
		if ($s === false) $s = array('uid' => 'ERR', 'gid' => 'ERR');
		echo '<td><nobr>' . t2h(username_from_uid($file['uid'])) . '</nobr></td>';
		echo '<td><nobr>' . t2h(groupname_from_gid($file['gid'])) . '</nobr></td>';

		if ($file['flags'] & FM_DIR)
		{
			$thissize = $file['size'];
			echo '<td align=right>' . ($thissize === '' ? '&nbsp;' : format_number($thissize)) . '&nbsp;</td>';
			echo '<td>' . date('d.m.Y H:i:s', $file['time']) . '</td>';
		}
		else
		{
			$thissize = $file['size'];
			echo '<td align=right>' . format_number($thissize) . '&nbsp;</td>';
			echo '<td>' . date('d.m.Y H:i:s', $file['time']) . '</td>';
		}
		echo '</tr>';
	}
}

echo "</table>
</p>
</body>
</html>";

?>
