<?php
// BlackBoard - PHP Internet Newsboard System
// Copyright 2003-4 by Yves Goergen
// Homepage: http://blackboard.unclassified.de
// See docs/license.txt for distribution/license details
//
// forum.php
// Main page request program
// This file can be altered for embedded use of the newsboard system

// ----- BEGIN CONFIG AREA -----

// BlackBoard's lib path where public.lib.php is located
// If your board's library path is somewhere else or you change it here, you
//   also need to update it in the framework page library's include statement.
// Don't forget the trailing slash!
$path = 'bb_lib/';

// Framework's include file and begin_page/end_page function calls
#$CFG['frm_include'] = 'page.lib.php';
#$CFG['frm_beginpage'] = 'begin_page($title);';
#$CFG['frm_endpage'] = 'end_page();';
#$CFG['code_before_exit'] = 'end_page(false);';

// ----- END CONFIG AREA -----

// We don't want E_NOTICE output here!
ini_set('error_reporting', E_ALL & ~E_NOTICE);

// Framework's initialisation
if ($CFG['frm_include']) require_once($CFG['frm_include']);

// include BlackBoard's public library
require_once($path . 'public.lib.php');
require_once($libpath . 'common.lib.php');
ReadACL();

$request = $_REQUEST['req'];
if ($request == '') $request = 'main';

$title = '';


switch ($request)
{
	case 'abbc_help':
		$ME = $MODULE['abbc_help'];
		BeginHTML($T['abbc_help'], true);

		// Close window on ESC
		echo '<script type="text/javascript"><![CDATA[';
		echo 'function EscKey() { if (event.keyCode == 27) window.close(); } document.onkeypress = EscKey;';
		echo ']]></script>';

		require($libpath . 'lang/' . $LANG . '_abbc.php');
		UpdateStat('PageHits', 1);
		EndHTML(true);
		break;

	case 'textfilter':
		$ME = $MODULE['textfilter'];
		BeginHTML($T['textfilter'], true);

		// Close window on ESC
		echo '<script type="text/javascript"><![CDATA[';
		echo 'function EscKey() { if (event.keyCode == 27) window.close(); } document.onkeypress = EscKey;';
		echo ']]></script>';

		echo $T['textfilter_desc'];
		EndHTML(true);
		break;

	case 'admin':
		$ME = $MODULE['admin'];
		require($libpath . 'admin.inc.php');
		break;

	case 'checkdb':
		$ME = $MODULE['checkdb'];
		require($libpath . 'checkdb.inc.php');
		break;

	case 'cp':
		$ME = $MODULE['cp'];
		require($libpath . 'cp.inc.php');
		break;

	case 'derefer':
		$ME = $MODULE['derefer'];
		$url = trim($_GET['url']);
		if (strpos($url, ':') === false) $url = 'http://' . $url;
		addlog('goto_url ' . $url);

		// Security check: only allow (http, https, ftp, ftps, mailto, news) protocol here!
		if (strncasecmp($url, 'http:', 5) &&
		    strncasecmp($url, 'https:', 6) &&
		    strncasecmp($url, 'ftp:', 4) &&
		    strncasecmp($url, 'ftps:', 5) &&
		    strncasecmp($url, 'mailto:', 7) &&
		    strncasecmp($url, 'news:', 5))
		{
			// forbidden protocol!
			errorlog('forbidden link protocol detected!');

			BeginHTML('Forbidden link', true);
			echo '<p class="error">This link protocol is not allowed here!</p>';
			EndHTML(true);
			break;
		}

		ForwardHTML($url);
		break;

	case 'help':
		$ME = $MODULE['help'];

		BeginHTML($T['help']);

		addlog('help');
		if ($LoginUserID) $LoginUser->SetLastActivity(ULF_HELP);
		else SetGuestLastForum(ULF_HELP);

		require($libpath . 'lang/' . $LANG . '_help.php');
		UpdateStat('PageHits', 1);
		EndHTML();
		break;

	case 'main':
		$ME = $MODULE['main'];
		require($libpath . 'main.inc.php');
		break;

	case 'post':
		$ME = $MODULE['post'];
		require($libpath . 'post.inc.php');
		break;

	case 'register':
		$ME = $MODULE['register'];
		require($libpath . 'register.inc.php');
		break;

	case 'search':
		$ME = $MODULE['search'];
		require($libpath . 'search.inc.php');
		break;

	case 'setuser':
		$ME = $MODULE['setuser'];

		// Get forward URL
		switch ($_GET['module'])
		{
			case 'main':
				$url = $MODULE['main'] . PD . 'id=' . $_GET['id'] . '&amp;page=' . $_GET['page'] . sid();
				break;
			case 'thread':
				$url = $MODULE['thread'] . PD . 'id=' . $_GET['id'] . '&amp;page=' . $_GET['page'] . sid();
				break;
			case 'stat':
				$url = $MODULE['stat'] . PD . 'page=' . $_GET['page'] . sid();
				break;
			default:
				$url = $MODULE['main'] . sid(2);
		}

		// Set new User
		$name = trim($_POST['Name']);
		if (!isset($_POST['Name']))
		{
			term_session();
			ForwardHTML($url);
		}
		elseif ($name != '')
		{
			$user = new IUser;
			if ($userid = $user->FindByName($name))
			{
				if (create_session($userid, md5(trim($_POST['Password'])), $user->GetPassword(), $user->GetFlags() & 32))
				{
					// we were a "guest", so remove current session ID from the guests list
					$db->RemoveRecord('Session="' . session_id() . '"', 'Guests');

					$first_login = ($user->GetLastActivity() == 0);

					$user->SetLastLogin();

					addlog('login ' . $user->GetName() . ' (' . $user->GetID() . ')');
					if ($first_login)
						// forward this user to his CP at his first login
						ForwardHTML($MODULE['cp'] . PD . 'id=' . $userid . '&amp;item=2' . sid());
					else
						ForwardHTML($url);
				}
				else
				{
					addlog('login ' . $user->GetName() . ' error wrong_password');
					term_session();
					ForwardHTML($MODULE['main'] . PD . 'err=3&amp;userid=' . $user->GetID() . sid());
				}
			}
			else
			{
				addlog('login ' . $name . ' error unknown_user');
				term_session();
				ForwardHTML($MODULE['main'] . PD . 'err=2' . sid());
			}
		}
		else
		{
			term_session();
			ForwardHTML($MODULE['main'] . PD . 'err=1' . sid());
		}
		break;

	case 'showip':
		$ME = $MODULE['showip'];

		// Import request variables
		$postid = $_REQUEST['id'];
		if (!is_numeric($postid) || $postid < 0) $postid = 0;

		// Begin page
		BeginHTML($T['ip_host']);

		$post = new IPost;
		$post->Load($postid);

		$thread = new IThread;
		$thread->Load($post->GetThread());

		if (CheckRights('showip', $thread->GetForum(), $thread->GetID()))
		{
			echo $T['date'] . ': ' . FriendlyDate($post->GetDate(), 0, 7) . '<br>';
			echo $T['ip_address'] . ': ' . $post->GetIP() . '<br>';
			echo $T['hostname'] . ': ' . $post->GetHostname() . '<br>';

			addlog('show_ip for ' . $postid);
		}
		else
		{
			$error = $T['err_access_denied'];
			echo '<p class="error">' . $error . '</p>';
			errorlog($error);
			addlog('show_ip for ' . $postid . ' no_access');
		}

		UpdateStat('PageHits', 1);
		EndHTML();
		break;

	case 'stat':
		$ME = $MODULE['stat'];
		require($libpath . 'stat.inc.php');
		break;

	case 'thread':
		$ME = $MODULE['thread'];
		require($libpath . 'thread.inc.php');
		break;

	case 'users':
		$ME = $MODULE['users'];
		require($libpath . 'users.inc.php');
		break;

	default:
		BeginHTML();
		echo '<p>Invalid request: ' . t2h($request) . '</p>';
		EndHTML();
}

?>